EMPTY = 0  # black
INACTIVE = 1  # grey
RECORDING = 5  # red
PLAYING = 21  # green
STOPPED = 43  # navy
CUT = 13  # cut mode - yellow
ERASE = 84  # orange
PULSE = 53  # pink
TAP = 37  # cyan
GREEN = [123, 23, 64, 22, 76, 87, 21, 122]

NUMBER_LOOPS = 32
NUMBER_SCENES = 8
